/*
 * Copyright (c) 1987 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that: (1) source distributions retain this entire copyright
 * notice and comment, and (2) distributions including binaries display
 * the following acknowledgement:  ``This product includes software
 * developed by the University of California, Berkeley and its contributors''
 * in the documentation or other materials provided with the distribution
 * and in all advertising materials mentioning features or use of this
 * software. Neither the name of the University nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#if defined(LIBC_SCCS) && !defined(lint)
static char sccsid[] = "@(#)setenv.c	5.4 (Berkeley) 6/1/90";
#endif /* LIBC_SCCS and not lint */

#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
/* extern char *malloc(); */

#ifdef BSD4_2
#define memcpy(a,b,c) bcopy(b,a,c)
#endif

int setenv (register const char *name, register const char *value, int rewrite);
void unsetenv(const char *name);
char *_findenv(register const char *name, int *offset);

/*
 * setenv --
 *	Set the value of the environmental variable "name" to be
 *	"value".  If rewrite is set, replace any current value.
 */

#ifdef __linux__	/* 01/18/98 katayama@rd.nacsis.ac.jp */
extern char **environ;
#else
extern const char **environ;
#endif

int
setenv(name, value, rewrite)
	register const char *name, *value;
	int rewrite;
{
	static int alloced;			/* if allocated space before */
	register char *C;
	register const char *D;
	int l_value, offset;

	if (*value == '=')			/* no `=' in value */
		++value;
	l_value = strlen(value);
	if ((C = _findenv(name, &offset))) {	/* find if already exists */
		if (!rewrite) {
		        fprintf(stderr,"setenv: returning\n");
			return(0);
		      }
	}
	else {					/* create new slot */
		register int	cnt;
		register const char	**P;

		for (P = environ, cnt = 0; *P; ++P, ++cnt);
		if (alloced) {			/* just increase size */
			environ = (const char **)realloc((char *)environ,
			    (size_t)(sizeof(char *) * (cnt + 2)));
			if (!environ) {
			  return(-1);
			}
		}
		else {				/* get new space */
			alloced = cnt+1;	/* copy old entries into it */
			P = (const char **)malloc((size_t)(sizeof(char *) *
			    (cnt + 2)));
			if (!P) {
			  return(-1);
			}
			memcpy(P, environ, cnt * sizeof(char *));
			environ = P;
		}
		environ[cnt] = malloc(1); /* So that can be freed later */
		environ[cnt + 1] = NULL;
		offset = cnt;
	}
	if(!name) {
	  return -1;
	}

	for (D = name; *D && *D != '='; ++D);	/* no `=' in name */

	if (!(C =			/* name + `=' + value */
	    malloc((size_t)((int)(D - name) + l_value + 2)))) {
		return(-1);
	      }
	if (offset >= alloced-1) free((/*discard const*/char*)environ[offset]);

	environ[offset] = C;

	for (; (*C = *name++) && *C != '='; ++C);
	for (*C++ = '='; (*C++ = *value++););
	return(0);
}

/*
 * unsetenv(name) --
 *	Delete environmental variable "name".
 */
void
unsetenv(name)
	const char	*name;
{
	register const char **P;
	int offset;

	while (_findenv(name, &offset))		/* if set multiple times */
		for (P = &environ[offset];; ++P)
			if (!(*P = *(P + 1)))
				break;
}
